#include <vector>
#include <iostream>
#include <queue>
#include <map>
#include <fstream>
#include <sstream>
#include <string>
#include <cmath>
#include <set>
#include <algorithm>

using namespace std;

struct Map_Cell {
    char type;
};

struct Search_Cell {
    pair<int, int> pos;
    int g;  // 从起点到当前节点的实际代价
    int h;  // 当前节点到目标的启发式估计代价
    int f() const { return g + h; }
    set<pair<int, int>> collected_points;  // 已经经过的配送点集合
    char direction = 'S';  // 移动方向，初始状态为 'S'
    Search_Cell* parent = nullptr;
};

// 优先队列的比较函数，用于选择 f 值最小的节点
struct CompareCell {
    bool operator()(const Search_Cell* a, const Search_Cell* b) const {
        return a->f() > b->f();
    }
};

// TODO: 设计一个合适的启发式函数
int heuristic(pair<int, int> current, pair<int, int> goal) {

    return 0;
}

void Astar_search(const string input_file, int &step_nums, string &way) {
    // 预处理部分
    ifstream file(input_file);
    if (!file.is_open()) {
        cout << "Error opening file!" << endl;
        return;
    }

    vector<string> input_map;
    string line;
    
    while (getline(file, line)) {
        input_map.push_back(line);
    }
    file.close();

    if (input_map.empty()) {
        cout << "Error: Empty map file!" << endl;
        return;
    }

    int M = input_map.size();
    int N = input_map[0].size();
    Map_Cell **Map = new Map_Cell *[M];

    pair<int, int> start, target;
    vector<pair<int, int>> special_points;

    // 读取地图
    for (int i = 0; i < M; i++) {
        Map[i] = new Map_Cell[N];
        for (int j = 0; j < N; j++) {
            Map[i][j].type = input_map[i][j];
            if (Map[i][j].type == 'S') start = {i, j};
            if (Map[i][j].type == 'T') target = {i, j};
            if (Map[i][j].type == 'B') special_points.push_back({i, j});
        }
    }

    vector<pair<int, int>> directions = {{0,1}, {0,-1}, {1,0}, {-1,0}};
    char dir_chars[] = {'R', 'L', 'D', 'U'};

    // 初始化 A* 搜索所需的 open_list（优先队列）。
    priority_queue<Search_Cell*, vector<Search_Cell*>, CompareCell> open_list;
    Search_Cell* initial = new Search_Cell();
    initial->pos = start;
    initial->g = 0;
    initial->h = heuristic(start, target);
    initial->direction = 'S';
    open_list.push(initial);

    // TODO: 实现 A* 搜索循环：
    while (!open_list.empty()) {

        // TODO: 检查是否已收集所有特殊点并到达终点, 若是则返回最优路径。(填充step_nums和way)

    }

    // 释放动态内存
    for(int i = 0; i < M; i++)
    {
        delete[] Map[i];
    }
    delete[] Map;
    while(!open_list.empty())
    {
        delete open_list.top();
        open_list.pop();
    }

    return;
}

void output(const string output_file, int &step_nums, string &way) {
    ofstream file(output_file);
    if (file.is_open()) {
        file << step_nums << endl;
        if (step_nums > 0) file << way << endl;
        file.close();
    }
    else {
        cerr << "Cannot open file: " << output_file << endl;
    }
    return;
}

int main() {
    clock_t start, end;
    start = clock();
    string input_base = "../input/input_";
    string output_base = "../output/output_";
    // 从input_1.txt到input_10.txt进行搜索，input_0.txt已用作演示
    for (int i = 1; i <= 10; i++) {
        int step_nums = -1;
        string way = "";
        Astar_search(input_base + to_string(i) + ".txt", step_nums, way);
        output(output_base + to_string(i) + "_Astar.txt", step_nums, way);
    }
    end = clock();
    cout << "time = " << double(end - start) / CLOCKS_PER_SEC << "s" << endl;
    return 0;
}