#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <cmath>
#include <set>
#include <map>
#include <algorithm>
using namespace std;

struct Map_Cell {
    char type;
};

struct Search_Cell {
    pair<int, int> pos;
    int g;  // 从起点到当前节点的实际代价
    set<pair<int, int>> collected_points;  // 已采集的配送点集合
    char direction = 'S';  // 移动方向，初始状态为 'S'
    Search_Cell* parent = nullptr;
};

// 辅助函数：根据当前 collected_points 计算 bitmask
// 举例来说，如果配送点有 3 个，对应的 bitmask 用 3 位二进制表示：若状态中采集了索引 0 和索引 2 的配送点，则 bitmask 为二进制 101，即十进制 5。
int getMask(const set<pair<int,int>>& s, const map<pair<int,int>, int>& specialIndex) {
    int mask = 0;
    for(auto p : s) {
        if(specialIndex.count(p)) {
            int idx = specialIndex.at(p);
            mask |= (1 << idx);
        }
    }
    return mask;
}

// 辅助函数：生成状态的唯一键 (仅沿当前路径检查去环)
string makeKey(const Search_Cell* cell, const map<pair<int,int>, int>& specialIndex) {
    int mask = getMask(cell->collected_points, specialIndex);
    return to_string(cell->pos.first) + "," + to_string(cell->pos.second) + "," + to_string(mask);
}

// 检查当前状态 key 是否在当前路径中出现（避免环路）  
bool inCurrentPath(const Search_Cell* node, const string& key, const map<pair<int,int>, int>& specialIndex) {
    const Search_Cell* cur = node;
    while(cur != nullptr) {
        if(makeKey(cur, specialIndex) == key)
            return true;
        cur = cur->parent;
    }
    return false;
}

// TODO: 递归的深度受限搜索 (DLS)
// 如果在当前深度限制内找到解，则返回对应的 Search_Cell*，否则返回 nullptr
Search_Cell* dls(Search_Cell* current, int depth, int limit, 
                 int all_mask, const pair<int,int>& target,
                 Map_Cell** Map, int M, int N,
                 const map<pair<int,int>, int>& specialIndex,
                 const vector<pair<int, int>>& directions, 
                 const char dir_chars[]) {
    int mask = getMask(current->collected_points, specialIndex);
    
}

void IDS_search(const string input_file, int &step_nums, string &way) {
    // 初始化
    ifstream file(input_file);
    if (!file.is_open()) {
        cout << "Error opening file!" << endl;
        return;
    }
    vector<string> input_map;
    string line;
    while (getline(file, line)) {
        input_map.push_back(line);
    }
    file.close();
    if (input_map.empty()) {
        cout << "Error: Empty map file!" << endl;
        return;
    }
    int M = input_map.size();
    int N = input_map[0].size();
    Map_Cell **Map = new Map_Cell *[M];
    pair<int, int> start, target;
    vector<pair<int, int>> special_points;
    // 读取地图
    for (int i = 0; i < M; i++) {
        Map[i] = new Map_Cell[N];
        for (int j = 0; j < N; j++) {
            Map[i][j].type = input_map[i][j];
            if (Map[i][j].type == 'S') start = {i, j};
            if (Map[i][j].type == 'T') target = {i, j};
            if (Map[i][j].type == 'B') special_points.push_back({i, j});
        }
    }
    // 建立配送点索引，用于将 collected_points 转换为 bitmask 状态
    map<pair<int, int>, int> specialIndex;
    for (int i = 0; i < special_points.size(); i++) {
        specialIndex[special_points[i]] = i;
    }
    int all_mask = (1 << special_points.size()) - 1; // 所有配送点均采集时的 bitmask

    // 定义四个方向
    vector<pair<int, int>> directions = { {0,1}, {0,-1}, {1,0}, {-1,0} };
    char dir_chars[] = {'R', 'L', 'D', 'U'};

    // 初始状态
    Search_Cell* initial = new Search_Cell();
    initial->pos = start;
    initial->g = 0;
    initial->parent = nullptr;
    // 初始时未采集任何配送点

    Search_Cell* solution = nullptr;
    // TODO: 迭代加深：从深度限制0开始逐步加深
    int max_limit = ;
    for (int limit = 0; limit <= max_limit; limit++) {
        
    }
    // 若找到解，则重建路径
    if (solution != nullptr) {
        step_nums = 0;
        string rev_path;
        Search_Cell* cell = solution;
        while(cell->parent != nullptr) {
            rev_path = cell->direction + rev_path;
            step_nums++;
            cell = cell->parent;
        }
        way = rev_path;
    } else {
        step_nums = -1;
        way = "";
    }
    // 释放初始状态（注意：如果 solution 不是 initial，其父节点链已被使用构造路径，简单释放 initial即可）
    delete initial;
    // 释放地图内存
    for (int i = 0; i < M; i++) {
        delete[] Map[i];
    }
    delete[] Map;
}

void output(const string output_file, int &step_nums, string &way) {
    ofstream file(output_file);
    if (file.is_open()) {
        file << step_nums << endl;
        if (step_nums > 0)
            file << way << endl;
        file.close();
    } else {
        cerr << "Can not open file: " << output_file << endl;
    }
}

int main() {
    clock_t start_time, end_time;
    start_time = clock();
    string input_base = "../input/input_";
    string output_base = "../output/output_";
    for (int i = 0; i <= 0; i++) {
        int step_nums = -1;
        string way = "";
        IDS_search(input_base + to_string(i) + ".txt", step_nums, way);
        output(output_base + to_string(i) + "_ids.txt", step_nums, way);
    }
    end_time = clock();
    cout << "time = " << double(end_time - start_time) / CLOCKS_PER_SEC << "s" << endl;
    return 0;
}